@@ -85,9 +85,10 @@ editor_button_press_event_cb (GtkhtmlEditor *editor,
 	if (!in_selection) {
 		html_engine_disable_selection (html->engine);
 		html_engine_jump_at (html->engine, event->x, event->y);
-		gtkhtml_editor_update_context (editor);
 	}
 
+	gtkhtml_editor_update_context (editor);
+
 	gtk_menu_popup (
 		GTK_MENU (menu), NULL, NULL, NULL, NULL,
 		event->button, event->time);
@@ -239,12 +240,17 @@ editor_method_check_word (GtkHTML *html,
                           gpointer user_data)
 {
 	GtkhtmlEditor *editor = user_data;
-	gboolean correct = TRUE;
+	gboolean correct = FALSE;
 	GList *list;
 
 	list = editor->priv->active_spell_checkers;
 
-	while (list != NULL && correct) {
+	/* If no spell checkers are active, assume the word is correct. */
+	if (list == NULL)
+		return TRUE;
+
+	/* The word is correct if ANY active spell checker can verify it. */
+	while (list != NULL && !correct) {
 		GtkhtmlSpellChecker *checker = list->data;
 
 		correct = gtkhtml_spell_checker_check_word (checker, word, -1);
@@ -830,6 +836,9 @@ editor_init (GtkhtmlEditor *editor)
 	editor->vbox = g_object_ref_sink (gtk_vbox_new (FALSE, 0));
 	gtk_widget_show (editor->vbox);
 
+	bindtextdomain (GETTEXT_PACKAGE, LOCALEDIR);
+	bind_textdomain_codeset (GETTEXT_PACKAGE, "UTF-8");
+
 	gtkhtml_editor_private_init (editor);
 
 	html = gtkhtml_editor_get_html (editor);
@@ -1031,10 +1040,18 @@ gtkhtml_editor_set_changed (GtkhtmlEditor *editor,
 
 	g_return_if_fail (GTKHTML_IS_EDITOR (editor));
 
-	html = gtkhtml_editor_get_html (editor);
+	if (!changed) {
+		html = gtkhtml_editor_get_html (editor);
 
-	if (!changed)
-		html_engine_saved (html->engine);
+		/* XXX The NULL check was added to deal with a race in
+		 *     Evolution when saving a message to a remote Drafts
+		 *     folder (which is asynchronous) and then closing the
+		 *     editor (which is immediate).  The GtkHTML object
+		 *     may already be disposed by the time the Save Draft
+		 *     callback gets here.  See bug #553995 for details. */
+		if (html)
+			html_engine_saved (html->engine);
+	}
 
 	editor->priv->changed = changed;
 }
